package hexagon.graphics;

import hexagon.Camera;
import hexagon.model.RawModel;
import hexagon.model.TexturedModel;
import hexagon.world.entity.Entity;

import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL13;
import org.lwjgl.opengl.GL30;
import org.lwjgl.util.vector.Matrix4f;

public class Renderer {
	
	private Matrix4f projectionMatrix;
	
	public Renderer(Camera camera) {
		projectionMatrix = camera.projectionMatrix;
	}
	
	public void loadMatricesToShader(Shader shader) {
		shader.enable();
		shader.setUniformMat4("projectionMatrix", projectionMatrix);
	}
	
	public void prepare() {
		GL11.glClear(GL11.GL_COLOR_BUFFER_BIT | GL11.GL_DEPTH_BUFFER_BIT);
	}
	
	public void render(Entity entity, Shader shader, Camera camera) {
		TexturedModel model = entity.model;
		RawModel rawModel = model.getRawModel();
		GL30.glBindVertexArray(rawModel.getVaoID());
		GL13.glActiveTexture(GL13.GL_TEXTURE0);
		GL11.glBindTexture(GL11.GL_TEXTURE_2D, model.getTextureID());
		shader.setUniformMat4("transformationMatrix", entity.transfMatrix);
		GL11.glDrawElements(GL11.GL_TRIANGLES, rawModel.getVertexCount(), GL11.GL_UNSIGNED_INT, 0);
	}
	
}
