package hexagon.model;

import java.io.FileInputStream;
import java.io.IOException;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;
import java.util.ArrayList;
import java.util.HashMap;

import org.lwjgl.BufferUtils;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL15;
import org.lwjgl.opengl.GL20;
import org.lwjgl.opengl.GL30;
import org.newdawn.slick.opengl.Texture;
import org.newdawn.slick.opengl.TextureLoader;

public class Loader {
	
	public static Loader theLoader = new Loader();
	
	private ArrayList<Integer> vaos = new ArrayList<Integer>();
	private ArrayList<Integer> vbos = new ArrayList<Integer>();
	private ArrayList<Integer> textures = new ArrayList<Integer>();
	
	private HashMap<String, Integer> texIDs = new HashMap<String, Integer>();
	
	private Loader() {
		if (theLoader != null) theLoader = this;
	}
	
	public RawModel loadToVAO(float[] positions, float[] texCoords, int[] indices) {
		int vaoID = createVAO();
		loadIBO(indices);
		storeData(0, 3, positions);
		storeData(1, 2, texCoords);
		unbindVAO();
		return new RawModel(vaoID, indices.length);
	}
	
	public RawModel loadToVAO(float[] positions) {
		int vaoID = createVAO();
		storeData(0, 2, positions);
		unbindVAO();
		return new RawModel(vaoID, positions.length / 2);
	}
	
	private int createVAO() {
		int vaoID = GL30.glGenVertexArrays();
		vaos.add(vaoID);
		GL30.glBindVertexArray(vaoID);
		return vaoID;
	}
	
	public int loadTexture(String name) {
		Integer id = texIDs.get(name);
		if (id != null) return id;
		
		Texture texture = null;
		try {
			texture = TextureLoader.getTexture("PNG", new FileInputStream("res/" + name + ".png"));
		} catch (IOException e) {
			e.printStackTrace();
		}
		int texID = texture.getTextureID();
		textures.add(texID);
		texIDs.put(name, texID);
		return texID;
	}
	
	private void storeData(int attrNumber, int coordinateSize, float[] data) {
		int vboID = GL15.glGenBuffers();
		vbos.add(vboID);
		GL15.glBindBuffer(GL15.GL_ARRAY_BUFFER, vboID);
		FloatBuffer buffer = storeInFloatBuffer(data);
		GL15.glBufferData(GL15.GL_ARRAY_BUFFER, buffer, GL15.GL_STATIC_DRAW);
		GL20.glVertexAttribPointer(attrNumber, coordinateSize, GL11.GL_FLOAT, false, 0, 0);
		GL15.glBindBuffer(GL15.GL_ARRAY_BUFFER, 0);
		GL20.glEnableVertexAttribArray(attrNumber);
	}
	
	private void unbindVAO() {
		GL30.glBindVertexArray(0);
	}
	
	private void loadIBO(int[] ids) {
		int vbo = GL15.glGenBuffers();
		vbos.add(vbo);
		GL15.glBindBuffer(GL15.GL_ELEMENT_ARRAY_BUFFER, vbo);
		IntBuffer buffer = storeInIntBuffer(ids);
		GL15.glBufferData(GL15.GL_ELEMENT_ARRAY_BUFFER, buffer, GL15.GL_STATIC_DRAW);
	}
	
	private IntBuffer storeInIntBuffer(int[] data) {
		IntBuffer buffer = BufferUtils.createIntBuffer(data.length);
		buffer.put(data);
		buffer.flip();
		return buffer;
	}
	
	private FloatBuffer storeInFloatBuffer(float[] data) {
		FloatBuffer buffer = BufferUtils.createFloatBuffer(data.length);
		buffer.put(data);
		buffer.flip();
		return buffer;
	}
	
	public void cleanUp() {
		for (int i : vaos)
			GL30.glDeleteVertexArrays(i);
		for (int i : vbos)
			GL15.glDeleteBuffers(i);
		for (int i : textures)
			GL11.glDeleteTextures(i);
	}
	
}
