package hexagon.util;

import hexagon.Camera;
import hexagon.model.HexagonModel;
import hexagon.world.Chunk;
import hexagon.world.ChunkRing;
import hexagon.world.World;
import hexagon.world.entity.Hexagon;

import org.lwjgl.util.vector.Matrix4f;
import org.lwjgl.util.vector.Vector2f;
import org.lwjgl.util.vector.Vector3f;

public class MathUtil {
	
	private MathUtil() {}
	
	public static Matrix4f transformationMatrix(Vector2f pos, Vector2f scale, Matrix4f matr) {
		if (matr == null) matr = new Matrix4f();
		else matr.setIdentity();
		Matrix4f.translate(pos, matr, matr);
		Matrix4f.scale(new Vector3f(scale.x, scale.y, 1), matr, matr);
		return matr;
	}
	
	public static Matrix4f transformationMatrix(Vector3f pos, Vector3f rotation, float scale, Matrix4f matr) {
		if (matr == null) matr = new Matrix4f();
		else matr.setIdentity();
		Matrix4f.translate(pos, matr, matr);
		float len = rotation.length();
		Matrix4f.rotate((float) (Math.PI / 2) - rotation.x, new Vector3f(1, 0, 0), matr, matr);
//		Matrix4f.rotate(len, new Vector3f(rotation.x / len, rotation.y / len, rotation.z / len), matr, matr);
		Matrix4f.scale(new Vector3f(scale, scale, scale), matr, matr);
		return matr;
	}
	
	public static void initHexagon(Hexagon hex, int x, int y, int z) {
		hex.rotation.x = (float) ((x + 2 * z) * Math.PI / (ChunkRing.ringSize * Chunk.chunkSize));
		hex.position.setCyl((HexagonModel.x60 + 1) * x, (float) (World.radius * Math.pow(0.95, y)), hex.rotation.x);
		hex.updateTransfMatrix();
	}
	
	public static Matrix4f projectionMatrix(float fov, float aspect, float near, float far, Matrix4f matr) {
		if (matr == null) matr = new Matrix4f();
		else matr.setIdentity();
		float y_scale = (float) (1 / Math.tan(Math.PI / 180 * fov / 2) * aspect);
		float frustLen = far - near;
		
		matr.m00 = y_scale / aspect;
		matr.m11 = y_scale;
		matr.m22 = -(near + far) / frustLen;
		matr.m23 = -1;
		matr.m32 = -2 * near * far / frustLen;
		matr.m33 = 0;
		
		return matr;
	}
	
	public static Matrix4f viewMatrix(Camera camera, Matrix4f matr) {
		if (matr == null) matr = new Matrix4f();
		else matr.setIdentity();
		Matrix4f.rotate(camera.rotation.z, new Vector3f(0, 0, 1), matr, matr);
		Matrix4f.rotate(camera.rotation.x, new Vector3f(1, 0, 0), matr, matr);
		Matrix4f.rotate(camera.rotation.y, new Vector3f(0, 1, 0), matr, matr);
		Matrix4f.rotate(camera.position.getCyl().z - (float) (Math.PI / 2), new Vector3f(1, 0, 0), matr, matr);
		Matrix4f.translate(camera.position.getEucl().negate(null), matr, matr);
		return matr;
	}
	
}
