package hexagon.world;

import hexagon.world.entity.Hexagon;

import java.util.function.Consumer;

public class Chunk {
	/** Number of hexagons in a chunk in one dimension (eg. 4 means 4x4 hexagons) */
	public static final int chunkSize = 16;
	
	protected World world;
	protected final HexagonColumn[] hexagonCols = new HexagonColumn[chunkSize * chunkSize];
	public final int x, y;
	
	public Chunk(int x, int y, Consumer<Chunk> generator, World world) {
		this.x = x;
		this.y = y;
		this.world = world;
		
		generator.accept(this);
	}
	
	public Hexagon getHexagon(int x, int y, int z) {
		return hexagonCols[x + z * chunkSize].getHexagon(y);
	}
	
	public void setHexagon(int x, int y, int z, Hexagon hexagon) {
		hexagonCols[x + z * chunkSize].getHexagon(y).onRemove();
		hexagonCols[x + z * chunkSize].setHexagon(hexagon, y);
	}
	
	public void updateHexagon(int x, int y, int z) {
		hexagonCols[x + z * chunkSize].getHexagon(y).onUpdate();
	}
	
	public void onRemove() {
		for (HexagonColumn c : hexagonCols)
			c.onRemove();
	}
	
}
