package hexagon.world;

import hexagon.Camera;
import hexagon.graphics.Renderer;
import hexagon.world.entity.Hexagon;

import java.util.Comparator;
import java.util.TreeMap;

public class HexagonColumn {
	
	private int x, z, yTop;
	private TreeMap<Integer, Hexagon> hexagons;
	private Chunk chunk;
	
	public HexagonColumn(int x, int z, Chunk chunk) {
		this.x = x;
		this.z = z;
		this.chunk = chunk;
		yTop = 0;
		hexagons = new TreeMap<>(Comparator.reverseOrder());
	}
	
	public void render(Renderer renderer, Camera camera) {
		for (Hexagon h : hexagons.values())
			if (h != null) h.render(renderer, camera);
	}
	
	public void setHexagon(Hexagon hex, int y) {
		Hexagon prev = hexagons.put(y, hex);
		if (prev != null) prev.onRemove();
	}
	
	public Hexagon getHexagon(int y) {
		return hexagons.get(y);
	}
	
	public void onRemove() {
		for (Hexagon h : hexagons.values())
			h.onRemove();
	}
	
}
