package hexagon.world;

import hexagon.Camera;
import hexagon.Main;
import hexagon.graphics.Renderer;
import hexagon.graphics.Shader;
import hexagon.model.HexagonModel;
import hexagon.world.entity.Hexagon;

import java.util.HashMap;
import java.util.function.BiFunction;
import java.util.function.Consumer;

import org.lwjgl.input.Mouse;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL13;
import org.lwjgl.opengl.GL30;
import org.lwjgl.util.vector.Vector3f;

public class World {
	
	public static final int viewDistance = 2; // temporary
	public static final float radius = (float) (HexagonModel.y60 / Math.sin(Math.PI / (Chunk.chunkSize * ChunkRing.ringSize)));
	
	private Hexagon hoverHexagon;
	private HashMap<Integer, ChunkRing> chunkRings = new HashMap<Integer, ChunkRing>();
	
	private BiFunction<Integer, Integer, Consumer<Chunk>> tempHexagonGenerator = (x, z) -> c -> {
		for (int i = 0; i < Chunk.chunkSize; i++) {
			for (int j = 0; j < Chunk.chunkSize; j++) {
				c.hexagonCols[i + j * Chunk.chunkSize] = new HexagonColumn(i, j, c);
				c.hexagonCols[i + j * Chunk.chunkSize].setHexagon(new Hexagon(x * Chunk.chunkSize + i, 0, z * Chunk.chunkSize + j, new Vector3f((float) i / Chunk.chunkSize, 0, (float) j / Chunk.chunkSize), this), 0);
				c.hexagonCols[i + j * Chunk.chunkSize].setHexagon(new Hexagon(x * Chunk.chunkSize + i, 1, z * Chunk.chunkSize + j, new Vector3f((float) i / Chunk.chunkSize, 0, (float) j / Chunk.chunkSize), this), 1);
				c.hexagonCols[i + j * Chunk.chunkSize].setHexagon(new Hexagon(x * Chunk.chunkSize + i, 2, z * Chunk.chunkSize + j, new Vector3f((float) i / Chunk.chunkSize, 0, (float) j / Chunk.chunkSize), this), 2);
			}
		}
	};
	
	public World() {
	}
	
	public void tick() {
		Vector3f mouse = Main.mousePicker.getCylinderPosition(radius); // Should really return cyl-coords
		
		if (mouse != null) {
			float mx = mouse.x, my = mouse.y, mz = mouse.z;
			if (mx != 0 || my != 0 || mz != 0) {
				Hexagon hex = null;
				float min = 0xffffff, temp;
				for (ChunkRing r : chunkRings.values()) {
					for (Chunk c : r.chunks) {
						if (c == null) continue;
						for (HexagonColumn hc : c.hexagonCols) {
							Hexagon h = hc.getHexagon(0);
							temp = h.distSq(mx, mz);
							if (temp < min) {
								min = temp;
								hex = h;
							}
						}
					}
				}
				if (hex != null) {
					if (hoverHexagon != hex) {
						if (hoverHexagon != null) hoverHexagon.deSelect();
						hex.select();
						hoverHexagon = hex;
					}
				} else if (hoverHexagon != null) hoverHexagon.deSelect();
			}
		} else if (hoverHexagon != null) hoverHexagon.deSelect();
		
		if (Mouse.isButtonDown(0)) {
			if (hoverHexagon != null) hoverHexagon.deSelect();
		}
	}
	
	public void render(Renderer renderer, Camera camera) {
		Shader.hexagon.enable();
		GL30.glBindVertexArray(Hexagon.model.getRawModel().getVaoID());
		GL13.glActiveTexture(GL13.GL_TEXTURE0);
		GL11.glBindTexture(GL11.GL_TEXTURE_2D, Hexagon.model.getTextureID());
		
		float angle = camera.position.getCyl().z;
		float zPos = angle * radius, delta = HexagonModel.y60 * Chunk.chunkSize;
//		System.out.println(angle);
		for (int camX = (int) (camera.position.getCyl().x / (HexagonModel.x60 + 1)) / Chunk.chunkSize, i = camX - viewDistance; i <= camX + viewDistance; i++) {
			for (int camZ = (int) (zPos - i * delta) / Chunk.chunkSize, j = camZ - viewDistance; j <= camZ + viewDistance; j++) {
//				Hexagon hex = getChunk(i, j).hexagonCols[0].getHexagon(0);
//				System.out.println((float) (Math.PI / 2 - hex.position.getCyl().z));
				for (HexagonColumn h : getChunk(i, j).hexagonCols) {
					h.render(renderer, camera);
				}
			}
		}
//		System.out.println("");
	}
	
	public Hexagon getHexagon(int x, int y, int z) {
		if (z < 0 || z >= Chunk.chunkSize * 4) return null;
		int cx = Math.floorDiv(x, Chunk.chunkSize), cz = z / Chunk.chunkSize;
		int mx = x % Chunk.chunkSize, mz = z % Chunk.chunkSize;
		
		Chunk c = getChunk(cx, cz);
		return c.getHexagon(mx, y, mz);
	}
	
	private ChunkRing getChunkRing(int x) {
		ChunkRing ring = chunkRings.get(x);
		
		if (ring != null) return ring;
		
		chunkRings.put(x, ring = new ChunkRing(x));
		return ring;
	}
	
	private ChunkRing setNewChunkRing(int x) {
		ChunkRing ring = chunkRings.get(x);
		
		if (ring != null) ring.onRemove();
		
		chunkRings.put(x, ring = new ChunkRing(x));
		return ring;
	}
	
	private Chunk getChunk(int x, int y) {
		y %= ChunkRing.ringSize;
		if (y < 0) y += ChunkRing.ringSize;
		
		ChunkRing ring = getChunkRing(x);
		
		Chunk chunk = ring.chunks[y];
		if (chunk == null) chunk = ring.chunks[y] = new Chunk(x, y, tempHexagonGenerator.apply(x, y), this);
		
		return chunk;
	}
	
	private Chunk setNewChunk(int x, int y) {
		y %= ChunkRing.ringSize;
		if (y < 0) y += ChunkRing.ringSize;
		
		ChunkRing ring = getChunkRing(x);
		Chunk chunk = ring.chunks[y];
		
		if (chunk != null) chunk.onRemove();
		chunk = ring.chunks[y] = new Chunk(x, y, tempHexagonGenerator.apply(x, y), this);
		
		return chunk;
	}
	
}
