package hexagon.world.entity;

import hexagon.Camera;
import hexagon.graphics.Renderer;
import hexagon.graphics.Shader;
import hexagon.model.HexagonModel;
import hexagon.model.Loader;
import hexagon.util.MathUtil;
import hexagon.util.Vec3f;
import hexagon.world.World;

import org.lwjgl.opengl.GL11;
import org.lwjgl.util.vector.Vector2f;
import org.lwjgl.util.vector.Vector3f;

public class Hexagon extends Entity {
	
	public static final Vector2f[] points = new Vector2f[6];
	public static HexagonModel model;
	
	private Vector3f colorTint, colSelected;
	private int x, y, z; // x: --		y: /		z: \
	private boolean isSelected = false;
	
	private World world;
	
	public Hexagon(int x, int y, int z, Vector3f color, World world) {
		super(model, new Vec3f(), new Vector3f(), 1f);
		MathUtil.initHexagon(this, x, y, z);
		this.x = x;
		this.z = z;
		this.y = -x - z;
		this.world = world;
		this.colorTint = color;
		colSelected = new Vector3f(color.x * (1 - color.x / 2), color.y * (1 - color.y / 2), color.z * (1 - color.z / 2));//new Color(0xffff00);
	}
	
	public void onUpdate() {
		
	}
	
	public void onRemove() {
		
	}
	
	public float distSq(float x, float v) {
		Vector3f cyl = position.getCyl();
		float xx = x - cyl.x, vv = v - cyl.z;
		vv *= cyl.y; // l = v*r
		return xx * xx + vv * vv;
	}
	
	public int getX() {
		return x;
	}
	
	public int getZ() {
		return z;
	}
	
	public boolean selected() {
		return isSelected;
	}
	
	public void select() {
		isSelected = true;
	}
	
	public void deSelect() {
		isSelected = false;
	}
	
//	private int distance(Hexagon hex) {
//		return Math.max(Math.max(Math.abs(x - hex.x), Math.abs(hex.y - y)), Math.abs(hex.z - z)) / 2;
//	}
	
	public void render(Renderer renderer, Camera camera) {
		if (isSelected) Shader.hexagon.setUniform1f("glow", 1.5f);
		Shader.hexagon.setUniform3f("colorTint", colorTint.x, colorTint.y, colorTint.z);
		Shader.hexagon.setUniformMat4("transformationMatrix", transfMatrix);
		
		GL11.glDrawElements(GL11.GL_TRIANGLES, model.getRawModel().getVertexCount(), GL11.GL_UNSIGNED_INT, 0);
		
		if (isSelected) Shader.hexagon.setUniform1f("glow", 1f);
	}
	
	public static void init() {
		for (int i = 0; i < 6; i++) {
			points[i] = new Vector2f((float) Math.cos(Math.PI * (i / 3d)), (float) Math.sin(Math.PI * (i / 3d)));
		}
		model = new HexagonModel(Loader.theLoader.loadTexture("test"));
	}
	
}
