package hexagon;

import java.io.File;

import org.lwjgl.input.Keyboard;
import org.lwjgl.input.Mouse;
import org.lwjgl.opengl.ContextAttribs;
import org.lwjgl.opengl.Display;
import org.lwjgl.opengl.DisplayMode;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.PixelFormat;
import org.newdawn.slick.util.Log;

import hexagon.graphics.Shader;
import hexagon.model.Loader;
import hexagon.util.MousePicker;
import hexagon.world.World;
import hexagon.world.entity.Hexagon;

public class Main {
	
	/**
	 *	Jag kanske skulle kunna göra tutorials på att programmera detta spel som ett gymnasiearbete! 
	 */
	
	public static final int width = 854, height = 480;
	public static MousePicker mousePicker;
	public static Main main;
	
	private World world;
	public Camera camera;
	
	private boolean F3_down = false;
	
	public Main() {
		main = this;
		initGL();
		initGame();
	}
	
	private void initGame() {
		Hexagon.init();
		Shader.init();
		
		camera = new Camera(70f, 16f / 9f, 0.02f, 1000f);
		camera.position.getCyl().r = (float) World.radius + 2;
		camera.position.updateEucl();
		camera.loadProjMatrToShader(Shader.block);
		Shader.block.setUniform1f("radius", (float) World.radius);
		mousePicker = new MousePicker(camera);
		
		world = new World();
	}
	
	private void tick() {
		if (Keyboard.isKeyDown(Keyboard.KEY_F3)) {
			if (!F3_down) {
				F3_down = true;
				Shader.init();
				camera.loadProjMatrToShader(Shader.block);
				Shader.block.setUniform1f("radius", (float) World.radius);
				Loader.theLoader.clearTextures();
				Hexagon.init();
				System.out.println("Reloaded textures and shaders.");
			}
		} else F3_down = false;
		
		camera.update();
		mousePicker.update();
		
		world.tick(camera);
	}
	
	private void loop() {
		long startTime;
		while (!Display.isCloseRequested()) {
			tick();
			
			GL11.glClear(GL11.GL_COLOR_BUFFER_BIT | GL11.GL_DEPTH_BUFFER_BIT);
			world.render(camera);
			
			int error = GL11.glGetError();
			if (error != GL11.GL_NO_ERROR) System.out.println("OpenGL error: " + error);
			
//			String debugInfo = "    C: (" + camera.position.getCyl().x + ", " + camera.position.getCyl().r + ", " + (float) (camera.position.getCyl().rv * 180 / Math.PI) + ")    E: (" + camera.position.getEucl().x + ", " + camera.position.getEucl().y + ", " + camera.position.getEucl().z + ")    R: (" + (float) (camera.rotation.x * 180 / Math.PI) + ", " + (float) (camera.rotation.y * 180 / Math.PI) + ", " + (float) (camera.rotation.z * 180 / Math.PI) + ")";
			Display.sync(60);
//			startTime = System.nanoTime();
			Display.update();
//			Display.setTitle("ms/update: " + (System.nanoTime() - startTime) / 1e6 + debugInfo);
		}
	}
	
	private void initGL() {
		try {
			Display.setDisplayMode(new DisplayMode(width, height));
			Display.create(new PixelFormat(), new ContextAttribs(3, 3));
			Display.setVSyncEnabled(true);
			Keyboard.create();
			Mouse.create();
		} catch (Exception e) {
			e.printStackTrace();
		}
		
//		GL11.glViewport(0, 0, width, height);
		GL11.glClearColor(0, 0, 0, 1);
		
		GL11.glEnable(GL11.GL_DEPTH_TEST);
		GL11.glDepthFunc(GL11.GL_LEQUAL);
//		GL11.glEnable(GL11.GL_TEXTURE_2D);
	}
	
	public static void destroy() {
		Loader.theLoader.cleanUp();
		Keyboard.destroy();
		Mouse.destroy();
		Display.destroy();
	}
	
	public static void main(String[] args) {
		File file = new File("lib/natives");
		if (!file.exists()) file = new File("lib/natives-" + (System.getProperty("os.name").toLowerCase().contains("win") ? "windows" : "linux"));
		System.setProperty("org.lwjgl.librarypath", file.getAbsolutePath());
		Log.setVerbose(false);
		
		new Main().loop();
		destroy();
	}
	
}
