package hexagon.world;

import org.lwjgl.util.vector.Vector3f;

import hexagon.graphics.BlockRenderer;

public class Block {
	
	public final int x, y, z;
	public final BlockColumn blockColumn;
	public Vector3f colorTint;
	private byte sidesToRender = 0;
	private boolean isSelected = false; // remove? Use hoverBlock in World instead?
	
	public Block(int x, int y, int z, BlockColumn blockColumn) {
		this.x = x;
		this.y = y;
		this.z = z;
		this.blockColumn = blockColumn;
		colorTint = new Vector3f(1f, 1f - (float) Math.random() * 0.5f, 1f);
		sidesToRender = (byte) ~0;
	}
	
	public void submit() {
		if (sidesToRender != 0) BlockRenderer.block.submit(this);
	}
	
	public boolean setSideRenderness(byte side, boolean render) {
		int prev = sidesToRender;
		if (render) sidesToRender |= 1 << side; // set to 1
		else sidesToRender &= ~(1 << side); // set to 0
		
		return sidesToRender != prev;
	}
	
	public int setSidesToRender(byte sides) {
		int diff = sides ^ sidesToRender;
		sidesToRender = sides;
		
		return diff;
	}
	
	public byte sidesToRender() {
		return sidesToRender;
	}
	
	public boolean selected() {
		return isSelected;
	}
	
	public void select() {
		isSelected = true;
	}
	
	public void deSelect() {
		isSelected = false;
	}
	
	public void onRemove() {}
	
	public void onUpdate() {}
	
}
