package hexagon.util;

import org.lwjgl.util.vector.Matrix4f;
import org.lwjgl.util.vector.Vector2f;
import org.lwjgl.util.vector.Vector3f;

import hexagon.Camera;
import hexagon.model.HexagonModel;
import hexagon.world.Chunk;
import hexagon.world.World;
import hexagon.world.entity.Hexagon;

public class MathUtil {
	
	public static final Vector3f unitX = new Vector3f(1, 0, 0);
	public static final Vector3f unitY = new Vector3f(0, 1, 0);
	public static final Vector3f unitZ = new Vector3f(0, 0, 1);
	
	private MathUtil() {}
	
	public static Matrix4f transformationMatrix(Vector2f pos, Vector2f scale, Matrix4f matr) {
		if (matr == null) matr = new Matrix4f();
		else matr.setIdentity();
		Matrix4f.translate(pos, matr, matr);
		Matrix4f.scale(new Vector3f(scale.x, scale.y, 1), matr, matr);
		return matr;
	}
	
	public static Matrix4f transformationMatrix(Vector3f pos, Vector3f rotation, float scale, float globalScale, Matrix4f matr) {
		if (matr == null) matr = new Matrix4f();
		else matr.setIdentity();
		Vector3f tempVec = new Vector3f();
		tempVec.set(globalScale, globalScale, globalScale);
		if (globalScale != 1) Matrix4f.scale(tempVec, matr, matr);
		Matrix4f.translate(pos, matr, matr);
		Matrix4f.rotate((float) (Math.PI / 2d) - rotation.x, unitX, matr, matr);
		Matrix4f.rotate(rotation.y, unitY, matr, matr);
		Matrix4f.rotate(rotation.z, unitZ, matr, matr);
		tempVec.set(scale, scale, scale);
		Matrix4f.scale(tempVec, matr, matr);
		return matr;
	}
	
	public static void initHexagon(Hexagon hex, int x, int y, int z) {
		double v = (x + 2 * z) * World.angleHalfHexagon;
		hex.rotation.x = (float) v;
		hex.position.setCyl((HexagonModel.x60 + 1) * x, World.radius, v);
	}
	
	public static Matrix4f projectionMatrix(float fov, float aspect, float near, float far, Matrix4f matr) {
		if (matr == null) matr = new Matrix4f();
		else matr.setIdentity();
		float y_scale = (float) (1 / Math.tan(Math.PI / 180 * fov / 2) * aspect);
		float frustLen = near - far;
		
		matr.m00 = y_scale / aspect;
		matr.m11 = y_scale;
		matr.m22 = (near + far) / frustLen;
		matr.m23 = -1;
		matr.m32 = 2 * near * far / frustLen;
		matr.m33 = 0;
		
		return matr;
	}
	
	public static Matrix4f viewMatrix(Camera camera, Matrix4f matr) {
		if (matr == null) matr = new Matrix4f();
		else matr.setIdentity();
		Matrix4f.rotate(camera.rotation.z, unitZ, matr, matr);
		Matrix4f.rotate(camera.rotation.x, unitX, matr, matr);
		Matrix4f.rotate(camera.rotation.y, unitY, matr, matr);
		Matrix4f.rotate((float) (camera.position.getCyl().rv / camera.position.getCyl().r - Math.PI / 2), unitX, matr, matr);
		Matrix4f.translate(camera.position.getEucl().negate(null), matr, matr);
		return matr;
	}
	
	public static void chunkCoordsToReal(Vector2f chunkCoords, Vector2f dest) {
		float xx = chunkCoords.x * (HexagonModel.x60 + 1);
		dest.y = fitChunkZ(chunkCoords.y + chunkCoords.x * 0.5f) * 2f * HexagonModel.y60;
		dest.x = xx;
	}
	
	public static void realCoordsToChunk(Vector2f realCoords, Vector2f dest) {
		float xx = realCoords.x / (HexagonModel.x60 + 1);
		dest.y = fitChunkZ((realCoords.y / HexagonModel.y60 - xx) / 2f);
		dest.x = xx;
	}
	
	public static int fitChunkZ(int z) {
		z %= Chunk.ringSize;
		if (z < 0) z += Chunk.ringSize;
		return z;
	}
	
	public static float fitChunkZ(float z) {
		z %= Chunk.ringSize;
		if (z < 0) z += Chunk.ringSize;
		return z;
	}
	
}
