package hexagon.world;

import org.lwjgl.util.vector.Vector3f;

import hexagon.Main;
import hexagon.graphics.BlockRenderer;
import hexagon.model.HexagonModel;
import hexagon.util.CylVec3;
import hexagon.util.MathUtil;
import hexagon.util.Vec3f;

public class Block {
	
	private static byte[][] neighborOffsets = {
			{0, -1, 0},
			{0, 1, 0},
			{1, 0, -1},
			{1, 0, 0},
			{0, 0, 1},
			{-1, 0, 1},
			{-1, 0, 0},
			{0, 0, -1}
	};
	
	public final int x, y, z;
	public final BlockColumn blockColumn;
	public Vector3f colorTint;
	private byte sidesToRender = 0;
	protected boolean needsUpdate = false;
	
	public Block(int x, int y, int z, BlockColumn blockColumn) {
		this.x = x;
		this.y = y;
		this.z = z;
		this.blockColumn = blockColumn;
//		colorTint = new Vector3f(1f, 1f - (float) Math.random() * 0.5f, 1f);
		sidesToRender = ~0;
	}
	
	public void submit() {
		float camX = Main.main.camera.position.getCyl().x;
		float mult = (float) Math.pow(1 - 1 / World.radius, y + 0.5f);
		CylVec3 temp = new CylVec3(x * (HexagonModel.x60 + 1) * mult + camX * (1 - mult), (float) (mult * World.radius), (float) ((x + 2 * z) * World.angleHalfHexagon));
//		if (x < 5 && x > -5 && z < 5) System.out.println(temp.x + " " + temp.r + " " + temp.rv);
		colorTint = MathUtil.closestBlock(new Vec3f(temp));
//		if (x < 5 && x > -5 && z < 5) System.out.println(colorTint + "\t" + x + " " + y + " " + z + "\n");
		Vector3f.sub(colorTint, new Vector3f(x, y + 0.5f, z), colorTint);
		colorTint.x = Math.abs(colorTint.x) * 100;
		colorTint.y = Math.abs(colorTint.y) * 100;
		colorTint.z = Math.abs(colorTint.z) * 100;
		if (sidesToRender != 0) BlockRenderer.block.submit(this);
	}
	
	public boolean setRenderSide(byte side, boolean render) {
		int prev = sidesToRender;
		if (render) sidesToRender |= 1 << side; // set to 1
		else sidesToRender &= ~(1 << side); // set to 0
		
		return sidesToRender != prev;
	}
	
	public int setSidesToRender(byte sides) {
		int diff = sides ^ sidesToRender;
		sidesToRender = sides;
		
		return diff;
	}
	
	public byte sidesToRender() {
		return sidesToRender;
	}
	
	public void onRemove() {}
	
	public void onUpdate(boolean silent) {
		World w = blockColumn.chunk.world;
		byte sides = 0;
		Block b;
		for (int i = 0; i < 8; i++) {
			b = w.getBlock(x + neighborOffsets[i][0], y + neighborOffsets[i][1], z + neighborOffsets[i][2]);
			if (b == null) sides |= 1 << i;
			else if ((sidesToRender & 1 << i) != 0) w.addBlockToUpdateList(b);
		}
		sidesToRender = sides;
	}
	
}
