package hexagon;

import java.util.function.Consumer;

import org.lwjgl.input.Keyboard;
import org.lwjgl.util.vector.Matrix4f;
import org.lwjgl.util.vector.Vector3f;

import hexagon.graphics.Shader;
import hexagon.util.CylVec3;
import hexagon.util.MathUtil;
import hexagon.util.Vec3f;
import hexagon.world.World;

public class Camera {
	
	public float fov, aspect, near, far;
	public Vec3f position;
	public Vector3f rotation;
	public Matrix4f projectionMatrix, viewMatrix;
	
	private Consumer<Shader> setViewMatrix = s -> {
		s.setUniformMat4("viewMatrix", viewMatrix);
		s.setUniform1f("camX", position.getCyl().x);
	};
	private Consumer<Shader> setProjMatrix = s -> s.setUniformMat4("projectionMatrix", projectionMatrix);
	
	public Camera(float fov, float aspect, float near, float far) {
		this.fov = fov;
		this.aspect = aspect;
		this.near = near;
		this.far = far;
		position = new Vec3f();
		rotation = new Vector3f();
		projectionMatrix = MathUtil.projectionMatrix(fov, aspect, near, far, null);
		viewMatrix = MathUtil.viewMatrix(this, null);
	}
	
	public void update() {
		CylVec3 cylPos = position.getCyl();
		float speed = Keyboard.isKeyDown(Keyboard.KEY_LCONTROL) ? 0.01f : Keyboard.isKeyDown(Keyboard.KEY_RCONTROL) ? 10f : 0.5f, rSpeed = 0.05f;
		speed *= cylPos.r / World.radius;
		if (Keyboard.isKeyDown(Keyboard.KEY_W)) {
			cylPos.rv += (float) (Math.cos(rotation.y) * speed);
			cylPos.x += (float) (Math.sin(rotation.y) * speed);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_S)) {
			cylPos.rv -= (float) (Math.cos(rotation.y) * speed);
			cylPos.x -= (float) (Math.sin(rotation.y) * speed);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_D)) {
			cylPos.x += (float) (Math.cos(rotation.y) * speed);
			cylPos.rv -= (float) (Math.sin(rotation.y) * speed);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_A)) {
			cylPos.x -= (float) (Math.cos(rotation.y) * speed);
			cylPos.rv += (float) (Math.sin(rotation.y) * speed);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_SPACE)) {
			cylPos.rv /= cylPos.r / (cylPos.r += speed);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_LSHIFT)) {
			cylPos.rv /= cylPos.r / (cylPos.r -= speed);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_UP)) {
			rotation.x -= rSpeed;
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_DOWN)) {
			rotation.x += rSpeed;
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_LEFT)) {
			rotation.y -= rSpeed;
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_RIGHT)) {
			rotation.y += rSpeed;
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_PRIOR)) {
			rotation.z -= rSpeed;
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_NEXT)) {
			rotation.z += rSpeed;
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_DELETE)) {
			if (Keyboard.isKeyDown(Keyboard.KEY_P)) {
				cylPos.x = 0;
				cylPos.rv = 0;
				cylPos.r = (float) World.radius + 2;
			}
			if (Keyboard.isKeyDown(Keyboard.KEY_R)) {
				rotation.x = rotation.y = rotation.z = 0;
			}
		}
		position.updateEucl();
		MathUtil.viewMatrix(this, viewMatrix);
		Shader.forEachShader(setViewMatrix);
	}
	
	public void loadProjMatrToShader(Shader shader) {
		shader.enable();
		setProjMatrix.accept(shader);
	}
	
}
