#version 400 core
#define PId3 3.14159265 / 3

layout (triangles) in;
layout (triangle_strip, max_vertices = 3) out;

in EvalOut {
	vec2 texCoords;
	vec3 colorTint;
} evalOut[];

out FragIn {
	vec2 texCoords;
	vec3 colorTint;
} fragIn;

uniform mat4 projectionMatrix;
uniform mat4 viewMatrix;
uniform float radius;
uniform float camX;

const float y60 = sin(PId3);
const float aHalfHex = y60 / radius;

vec4 transformVector(vec4 vector) {
	float v1 = vector.z * aHalfHex;
	float mult = pow(1 - 1 / radius, vector.y);
	return vec4((vector.x - camX) * mult + camX, sin(v1) * radius * mult, cos(v1) * radius * mult, 1);
}

void main() {
	mat4 matrix = projectionMatrix * viewMatrix;
	
	fragIn.texCoords = evalOut[0].texCoords;
	fragIn.colorTint = evalOut[0].colorTint;
	gl_Position = matrix * transformVector(gl_in[0].gl_Position);
	
	fragIn.texCoords = evalOut[1].texCoords;
	fragIn.colorTint = evalOut[1].colorTint;
	gl_Position = matrix * transformVector(gl_in[1].gl_Position);
	
	fragIn.texCoords = evalOut[2].texCoords;
	fragIn.colorTint = evalOut[2].colorTint;
	gl_Position = matrix * transformVector(gl_in[2].gl_Position);
}