package hexagon;

import hexagon.graphics.Shader;
import hexagon.util.MathUtil;
import hexagon.util.Vec3f;
import hexagon.world.World;

import java.util.function.Consumer;

import org.lwjgl.input.Keyboard;
import org.lwjgl.util.vector.Matrix4f;
import org.lwjgl.util.vector.Vector3f;

public class Camera {
	
	public float fov, aspect, near, far;
	public Vec3f position;
	public Vector3f rotation;
	public Matrix4f projectionMatrix, viewMatrix;
	
	private Consumer<Shader> setViewMatrix = s -> s.setUniformMat4("viewMatrix", viewMatrix);
	
	public Camera(float fov, float aspect, float near, float far) {
		this.fov = fov;
		this.aspect = aspect;
		this.near = near;
		this.far = far;
		position = new Vec3f();
		rotation = new Vector3f();
		projectionMatrix = MathUtil.projectionMatrix(fov, aspect, near, far, null);
		viewMatrix = MathUtil.viewMatrix(this, null);
	}
	
	public void update() {
		Vector3f cylPos = position.getCyl();
		float speed = Keyboard.isKeyDown(Keyboard.KEY_LCONTROL) ? 0.01f : Keyboard.isKeyDown(Keyboard.KEY_RCONTROL) ? 10f : 0.5f, rSpeed = 0.05f;
		speed *= cylPos.y / World.radius;
		if (Keyboard.isKeyDown(Keyboard.KEY_W)) {
			cylPos.z += (float) (Math.cos(rotation.y) * speed) / cylPos.y; // l = v*r
			cylPos.x += (float) (Math.sin(rotation.y) * speed);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_S)) {
			cylPos.z -= (float) (Math.cos(rotation.y) * speed) / cylPos.y;
			cylPos.x -= (float) (Math.sin(rotation.y) * speed);
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_D)) {
			cylPos.x += (float) (Math.cos(rotation.y) * speed);
			cylPos.z -= (float) (Math.sin(rotation.y) * speed) / cylPos.y;
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_A)) {
			cylPos.x -= (float) (Math.cos(rotation.y) * speed);
			cylPos.z += (float) (Math.sin(rotation.y) * speed) / cylPos.y;
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_SPACE)) cylPos.y += speed;
		if (Keyboard.isKeyDown(Keyboard.KEY_LSHIFT)) cylPos.y -= speed;
		if (Keyboard.isKeyDown(Keyboard.KEY_UP)) {
			rotation.x -= rSpeed;//(float) Math.cos(relativeRotation.y) * rSpeed;
//			relativeRotation.z -= (float) Math.sin(relativeRotation.y) * rSpeed;
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_DOWN)) {
			rotation.x += rSpeed;//(float) Math.cos(relativeRotation.y) * rSpeed;
//			relativeRotation.z += (float) Math.sin(relativeRotation.y) * rSpeed;
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_LEFT)) {
			rotation.y -= rSpeed;
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_RIGHT)) {
			rotation.y += rSpeed;
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_PRIOR)) {
//			relativeRotation.x -= (float) Math.cos(relativeRotation.y) * rSpeed;
//			relativeRotation.z += (float) Math.sin(relativeRotation.y) * rSpeed;
			rotation.z -= rSpeed;//(float) Math.cos(relativeRotation.y) * rSpeed;
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_NEXT)) {
//			relativeRotation.x += (float) Math.cos(relativeRotation.y) * rSpeed;
//			relativeRotation.z -= (float) Math.sin(relativeRotation.y) * rSpeed;
			rotation.z += rSpeed;//(float) Math.cos(relativeRotation.y) * rSpeed;
		}
		if (Keyboard.isKeyDown(Keyboard.KEY_DELETE)) {
			if (Keyboard.isKeyDown(Keyboard.KEY_P)) {
				cylPos.x = 0;
				cylPos.z = 0;
				cylPos.y = World.radius + 2;
			}
			if (Keyboard.isKeyDown(Keyboard.KEY_R)) {
				rotation.x = rotation.y = rotation.z = 0;
			}
		}
		position.updateEucl();
		MathUtil.viewMatrix(this, viewMatrix);
		Shader.forEachShader(setViewMatrix);
	}
	
}
