package hexagon;

import java.io.File;

import org.lwjgl.input.Keyboard;
import org.lwjgl.input.Mouse;
import org.lwjgl.opengl.Display;
import org.lwjgl.opengl.DisplayMode;
import org.lwjgl.opengl.GL11;
import org.newdawn.slick.util.Log;

import hexagon.graphics.Renderer;
import hexagon.graphics.Shader;
import hexagon.model.Loader;
import hexagon.util.MousePicker;
import hexagon.world.World;
import hexagon.world.entity.Hexagon;

public class Main {
	
	public static final int width = 854, height = 480;
	public static MousePicker mousePicker;
	
	private World world;
	private Camera camera;
	
	private Renderer renderer;
	
	public Main() {
		initGL();
		initGame();
	}
	
	private void initGame() {
		Hexagon.init();
		Shader.init();
		
		camera = new Camera(70f, 16f / 9f, 0.02f, 1000f);
		camera.position.getCyl().y = World.radius + 2;
		camera.position.updateEucl();
		renderer = new Renderer(camera);
		mousePicker = new MousePicker(camera);
		
		world = new World();
	}
	
	private void tick() {
		camera.update();
		mousePicker.update();
		
		world.tick(camera);
	}
	
	private void loop() {
		renderer.loadMatricesToShader(Shader.hexagon);
		
//		ArrayList<GuiTexture> guis = new ArrayList<GuiTexture>();
//		guis.add(new GuiTexture(Loader.theLoader.loadTexture("test"), new Vector2f(0.5f, 0.5f), new Vector2f(0.25f, 0.25f)));
		
//		GuiRenderer guiRenderer = new GuiRenderer();
		
		long startTime;
		while (!Display.isCloseRequested()) {
			startTime = System.currentTimeMillis();
			tick();
			
			renderer.prepare();
			world.render(camera);
//			guiRenderer.render(guis);
			
			Display.setTitle("ms/update: " + (System.currentTimeMillis() - startTime) + "    x: " + camera.position.getCyl().x + "  r: " + camera.position.getCyl().y + "  v: " + (float) (camera.position.getCyl().z * 180 / Math.PI) + "    x: " + camera.position.getEucl().x + "  y: " + camera.position.getEucl().y + "  z: " + camera.position.getEucl().z + "    rx: " + (float) (camera.rotation.x * 180 / Math.PI) + "  ry: " + (float) (camera.rotation.y * 180 / Math.PI) + "  rz: " + (float) (camera.rotation.z * 180 / Math.PI));
			
			int error = GL11.glGetError();
			if (error != GL11.GL_NO_ERROR) System.out.println("OpenGL error: " + error);
			
			Display.sync(60);
			Display.update();
		}
	}
	
	private void initGL() {
		try {
			Display.setDisplayMode(new DisplayMode(width, height));
			Display.create();
			Display.setVSyncEnabled(true);
			Keyboard.create();
//			Mouse.create();
		} catch (Exception e) {
			e.printStackTrace();
		}
		
//		GL11.glViewport(0, 0, width, height);
		GL11.glClearColor(0, 0, 0, 1);
		
		GL11.glEnable(GL11.GL_DEPTH_TEST);
//		GL11.glEnable(GL11.GL_TEXTURE_2D);
	}
	
	public static void destroy() {
		Loader.theLoader.cleanUp();
		Keyboard.destroy();
		Mouse.destroy();
		Display.destroy();
	}
	
	public static void main(String[] args) {
		System.setProperty("org.lwjgl.librarypath", new File("lib/natives").getAbsolutePath());
		Log.setVerbose(false);
		
		new Main().loop();
		destroy();
	}
	
}
