package hexagon.graphics;

import java.nio.ByteBuffer;
import java.nio.ShortBuffer;

import org.lwjgl.BufferUtils;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL13;
import org.lwjgl.opengl.GL15;
import org.lwjgl.opengl.GL20;
import org.lwjgl.opengl.GL30;
import org.lwjgl.util.vector.Matrix4f;
import org.lwjgl.util.vector.Vector4f;

import hexagon.model.HexagonModel;
import hexagon.model.Loader;
import hexagon.world.Chunk;
import hexagon.world.entity.Hexagon;

public class HexagonRenderer {
	
	private static final int vertexSizeInBytes = 3 * 4 + 2 * 4 + 3 * 4;
	private static final int hexagonSizeInBytes = 6 * vertexSizeInBytes; // 120		Automation thanks!
	private static final int numberOfHexagons = Chunk.chunkSize * Chunk.chunkSize;
	private static final int bufferSize = numberOfHexagons * hexagonSizeInBytes;
	private static final int numIndices = numberOfHexagons * 12;
	private static Vector4f[] hexPos4f = new Vector4f[6];
	private int indicesUsed = 0;
	
	static {
		for (int i = 0; i < 6; i++)
			hexPos4f[i] = new Vector4f(HexagonModel.vertices[i * 3], HexagonModel.vertices[i * 3 + 1], HexagonModel.vertices[i * 3 + 2], 1);
	}
	
	private int vaoID, vboID, iboID;
	private ByteBuffer buffer = null;
	
	public HexagonRenderer() {
		vaoID = GL30.glGenVertexArrays();
		vboID = GL15.glGenBuffers();
		iboID = GL15.glGenBuffers();
		Loader.theLoader.vaos.add(vaoID);
		Loader.theLoader.vbos.add(vboID);
		Loader.theLoader.vbos.add(iboID);
		
		GL30.glBindVertexArray(vaoID);
		buffer = BufferUtils.createByteBuffer(bufferSize);
		GL15.glBindBuffer(GL15.GL_ARRAY_BUFFER, vboID);
		GL15.glBufferData(GL15.GL_ARRAY_BUFFER, bufferSize, GL15.GL_STREAM_DRAW);
		GL20.glEnableVertexAttribArray(0);
		GL20.glEnableVertexAttribArray(1);
		GL20.glEnableVertexAttribArray(2);
		GL20.glVertexAttribPointer(0, 3, GL11.GL_FLOAT, false, vertexSizeInBytes, 0); // position
		GL20.glVertexAttribPointer(1, 2, GL11.GL_FLOAT, false, vertexSizeInBytes, 3 * 4); // texCoords
		GL20.glVertexAttribPointer(2, 3, GL11.GL_FLOAT, false, vertexSizeInBytes, 5 * 4); // texCoords
		
		ShortBuffer indices = BufferUtils.createShortBuffer(numIndices);
		for (short i = 0; i < numberOfHexagons; i++) {
			for (short j = 0; j < 4; j++) {
				indices.put((short) (6 * i));
				indices.put((short) (6 * i + j + 1));
				indices.put((short) (6 * i + j + 2));
			}
		}
		indices.flip();
		
		GL15.glBindBuffer(GL15.GL_ELEMENT_ARRAY_BUFFER, iboID);
		GL15.glBufferData(GL15.GL_ELEMENT_ARRAY_BUFFER, indices, GL15.GL_STATIC_DRAW);
	}
	
	/**	Usage: begin, submit ..., end, render */
	public void submit(Hexagon hex) {
		if (buffer.remaining() < hexagonSizeInBytes) flush();
		
		Vector4f temp = new Vector4f();
//		buffer.mark();
		for (int i = 0; i < 6; i++) {
			Matrix4f.transform(hex.transfMatrix, hexPos4f[i], temp);
			buffer.putFloat(temp.x);
			buffer.putFloat(temp.y);
			buffer.putFloat(temp.z);
			buffer.putFloat(Hexagon.points[i].x);
			buffer.putFloat(Hexagon.points[i].y);
			buffer.putFloat(hex.colorTint.x);
			buffer.putFloat(hex.colorTint.y);
			buffer.putFloat(hex.colorTint.z);
		}
//		buffer.reset();
//		for (int i = 0; i < 6; i++) {
//			System.out.println(buffer.getFloat());
//			System.out.println(buffer.getFloat());
//			System.out.println(buffer.getFloat());
//			buffer.getFloat();
//			buffer.getFloat();
//			System.out.println();
//		}
//		System.out.println();
	}
	
	public void flush() {
		end();
		render();
		begin();
	}
	
	public void begin() {
		buffer.clear();
	}
	
	public void end() {
		indicesUsed = buffer.position() * 12 / hexagonSizeInBytes;
		buffer.flip();
		GL15.glBindBuffer(GL15.GL_ARRAY_BUFFER, vboID);
		GL15.glBufferSubData(GL15.GL_ARRAY_BUFFER, 0, buffer);
	}
	
	public void render() {
		GL30.glBindVertexArray(vaoID);
		GL15.glBindBuffer(GL15.GL_ARRAY_BUFFER, 0);
		GL15.glBindBuffer(GL15.GL_ELEMENT_ARRAY_BUFFER, iboID);
		GL13.glActiveTexture(GL13.GL_TEXTURE0);
		GL11.glBindTexture(GL11.GL_TEXTURE_2D, Hexagon.model.getTextureID());
		
		GL11.glDrawElements(GL11.GL_TRIANGLES, indicesUsed, GL11.GL_UNSIGNED_SHORT, 0);
	}
	
}
