package hexagon.util;

import org.lwjgl.input.Mouse;
import org.lwjgl.opengl.Display;
import org.lwjgl.util.vector.Matrix4f;
import org.lwjgl.util.vector.Vector3f;
import org.lwjgl.util.vector.Vector4f;

import hexagon.Camera;

public class MousePicker {
	
	private Vector3f currentRay;
	
	private Matrix4f projectionMatrix;
	private Matrix4f viewMatrix;
	private Camera camera;
	
	public MousePicker(Camera camera) {
		this.camera = camera;
		this.projectionMatrix = camera.projectionMatrix;
		viewMatrix = camera.viewMatrix;
	}
	
	public Vector3f getCurrentRay() {
		return currentRay;
	}
	
	public void update() {
		currentRay = calculateMouseRay();
	}
	
	private Vector3f calculateMouseRay() {
		return toWorldCoords(toEyeCoords(new Vector4f(Mouse.getX() * 2f / Display.getWidth() - 1, Mouse.getY() * 2f / Display.getHeight() - 1, -1f, 1f)));
	}
	
	private Vector3f toWorldCoords(Vector4f eyeCoords) {
		Matrix4f invView = Matrix4f.invert(viewMatrix, null);
		Vector4f rayWorld = Matrix4f.transform(invView, eyeCoords, null);
		Vector3f mouseRay = new Vector3f(rayWorld.x, rayWorld.y, rayWorld.z);
		mouseRay.normalise();
		return mouseRay;
	}
	
	private Vector4f toEyeCoords(Vector4f clipCoords) {
		Matrix4f invProj = Matrix4f.invert(projectionMatrix, null);
		Vector4f eyeCoords = Matrix4f.transform(invProj, clipCoords, null);
		return new Vector4f(eyeCoords.x, eyeCoords.y, -1f, 0f);
	}
	
	public Vector3f getCylinderPosition(float radius) {
		Vector3f camPos = camera.position.getEucl();
		float dy = currentRay.y;
		float dz = currentRay.z;
		if (dz == 0 && dy == 0) return null;
		float dx = currentRay.x;
		float len = (float) Math.hypot(dy, dz);
		dx /= len;
		dy /= len;
		dz /= len;
		
		float yy = camPos.y, zz = camPos.z;
		float b = dy * yy + dz * zz; // dot(dirVec, distVec)
		float c = yy * yy + zz * zz - radius * radius; // dot(distVec, distVec) - r^2
		float t, underSqrt = b * b - c;
		if (underSqrt < 0) return null;
		if (underSqrt == 0) t = -b;
		else {
			float sqrt = (float) Math.sqrt(underSqrt);
			float t1 = -b - sqrt, t2 = -b + sqrt;
			if (t1 > 0) t = t1;
			else if (t2 > 0) t = t2;
			else return null;
//			System.out.println(t1 + " " + t2 + " " + t);
		}
		
		return new Vector3f(camPos.x + dx * t, camPos.y + dy * t, camPos.z + dz * t);
	}
	
}
