package hexagon.world;

import java.util.Comparator;
import java.util.TreeMap;

import org.lwjgl.util.vector.Matrix4f;
import org.lwjgl.util.vector.Vector3f;
import org.lwjgl.util.vector.Vector4f;

import hexagon.model.HexagonModel;
import hexagon.world.entity.Hexagon;

public class BlockColumn {
	
	private int x, z, yTop;
	public final Vector3f[] points = new Vector3f[6];
	protected TreeMap<Integer, Block> blocks;
	private Chunk chunk;
	
	public BlockColumn(int x, int z, Chunk chunk) {
		this.x = x;
		this.z = z;
		this.chunk = chunk;
		yTop = 0;
		blocks = new TreeMap<>(Comparator.reverseOrder());
		
		Matrix4f matr = new Hexagon(chunk.x * Chunk.chunkSize + x, 0, chunk.z * Chunk.chunkSize + z, null, null).transfMatrix;
		Vector4f temp = new Vector4f();
		for (int i = 0; i < 6; i++) {
			temp.set(HexagonModel.vertices[3 * i], HexagonModel.vertices[3 * i + 1], HexagonModel.vertices[3 * i + 2], 1);
			Matrix4f.transform(matr, temp, temp);
			points[i] = new Vector3f(temp.x, temp.y, temp.z);
		}
	}
	
	public void setBlock(Block block, int y) {
		Block prev = blocks.put(y, block);
		if (prev != null) prev.onRemove();
	}
	
	public Block getBlock(int y) {
		return blocks.get(y);
	}
	
	public void onRemove() {
		for (Block b : blocks.values())
			b.onRemove();
		blocks.clear();
	}
	
}
