#version 330 core

layout (points) in;
layout (triangle_strip, max_vertices = 36) out;

in VertOut {
	vec3 colorTint;
	int sides;
} vertOut[];

out FragIn {
	vec2 texCoords;
	vec3 colorTint;
} fragIn;

uniform mat4 projectionMatrix;
uniform mat4 viewMatrix;
uniform float radius;

const float PI = 3.14159265;
const float PI2 = 2 * PI;

void main() {
	fragIn.colorTint = vertOut[0].colorTint;
	mat4 matrix = projectionMatrix * viewMatrix;
	vec4 pos = gl_in[0].gl_Position;
	int sides = vertOut[0].sides;
	float camX = -viewMatrix[0][3];
	
	vec3 pos_map[12];
	vec2 tex_map[12];
	int index_map[12];
	
	float y60 = sin(PI / 3), x60 = cos(PI / 3);
	float angleHalfHexagon = y60 / radius; // radius / y60 = 2 * ringSize * chunkSize / (2 * PI)
	float mult1 = pow(1 - 1 / radius, pos.y), mult2 = pow(1 - 1 / radius, pos.y + 1);
	float xpos = (x60 + 1) * pos.x;
	
	for (int i = 1, idx = 0; i > -2; i--) { // 1, 0, -1
		float v1 = ((pos.x + 2 * pos.z) + i) * angleHalfHexagon;
		float v2 = PI / 3 * i;
		float y = sin(v1) * radius, z = cos(v1) * radius, cos2 = cos(v2), sin2 = sin(v2);
		
		for (int j = 0; j < 2; j++) { // 0, 1
			float mult = j == 0 ? mult1 : mult2;
			int index = (6 + i) % 6 + 6 * j;
			
			index_map[idx + 6 * j] = index;
			pos_map[index] = vec3(xpos + cos2, y, z) * mult;
			pos_map[index].x += camX * (1.0 - mult);
			tex_map[index] = vec2(cos2, sin2);
			
			index = (6 + 3 - i) % 6 + 6 * j;
			
			index_map[idx + 6 * j + 1] = index;
			pos_map[index] = vec3(xpos - cos2, y, z) * mult;
			pos_map[index].x += camX * (1.0 - mult);
			tex_map[index] = vec2(-cos2, sin2);
		}
		idx += 2;
	}
	
	if ((sides & 1) != 0) {
		for (int i = 0; i < 6; i++) {
			int index = index_map[i];
			gl_Position = matrix * vec4(pos_map[index], 1);
			fragIn.texCoords.x = (1 + tex_map[index].x) / 2;
			fragIn.texCoords.y = (1 - tex_map[index].y) / 2;
			EmitVertex();
		}
		EndPrimitive();
	}
	if ((sides & 1 << 1) != 0) {
		for (int i = 0; i < 6; i++) {
			int i1 = index_map[11 - i], i2 = index_map[6 + i];
			gl_Position = matrix * vec4(pos_map[i1], 1);
			fragIn.texCoords.x = (1 - tex_map[i2].x) / 2;
			fragIn.texCoords.y = (1 - tex_map[i2].y) / 2;
			EmitVertex();
		}
		EndPrimitive();
	}
	for (int j = 0; j < 6; j++) {
		if ((sides & 1 << j + 2) != 0) {
			for (int i = 0; i < 4; i++) {
				int index = (6 + j - i % 2) % 6 + 6 * (i / 2);
				gl_Position = matrix * vec4(pos_map[index], 1);
				fragIn.texCoords.x = 1 - i % 2;
				fragIn.texCoords.y = i / 2;
				EmitVertex();
			}
			EndPrimitive();
		}
	}
}