package hexagon.graphics;

import java.nio.ByteBuffer;
import java.nio.ShortBuffer;
import java.util.function.BiConsumer;

import org.lwjgl.BufferUtils;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL13;
import org.lwjgl.opengl.GL15;
import org.lwjgl.opengl.GL20;
import org.lwjgl.opengl.GL30;

import hexagon.Main;
import hexagon.model.Loader;
import hexagon.world.Block;
import hexagon.world.BlockSide;
import hexagon.world.entity.Hexagon;

public class BlockRenderer<T> {
	
	public static final BlockRenderer<Block> block;
	public static final BlockRenderer<Hexagon> hexagon;
	public static final BlockRenderer<BlockSide> blockSide;
	
	static {
		block = new BlockRenderer<>(3 * 4 + 3 * 4 + 1 * 4, 1, 10000, (b, r) -> {
			r.buffer.putInt(b.x).putInt(b.y).putInt(b.z);
			r.buffer.putFloat(b.up.colorTint.x);
			r.buffer.putFloat(b.up.colorTint.y);
			r.buffer.putFloat(b.up.colorTint.z);
			r.buffer.putInt(b.sidesToRender());
		});
		hexagon = new BlockRenderer<>(3 * 4 + 2 * 4 + 3 * 4, 6, 10000, (h, r) -> {
			for (int i = 0; i < r.verticesPerObject; i++) {
//				r.buffer.putFloat(4.65783f).putFloat(4.65483f).putFloat(4.65783f).putFloat(4.65883f).putFloat(4.65083f).putFloat(4.65783f).putFloat(4.62783f).putFloat(2.65783f);
//				r.buffer.putInt(Float.floatToRawIntBits(4.65783f)).putInt(Float.floatToRawIntBits(4.65483f)).putInt(Float.floatToRawIntBits(4.65783f)).putInt(Float.floatToRawIntBits(4.65883f)).putInt(Float.floatToRawIntBits(4.65083f)).putInt(Float.floatToRawIntBits(4.65783f)).putInt(Float.floatToRawIntBits(4.62783f)).putInt(Float.floatToRawIntBits(2.65783f));
				r.buffer.putFloat(h.block.blockColumn.points[i].x * h.globalScale + Main.main.camera.position.getEucl().x * (1f - h.globalScale));
				r.buffer.putFloat(h.block.blockColumn.points[i].y * h.globalScale);
				r.buffer.putFloat(h.block.blockColumn.points[i].z * h.globalScale);
				r.buffer.putFloat(Hexagon.points[i].x);
				r.buffer.putFloat(Hexagon.points[i].y);
				r.buffer.putFloat(h.colorTint.x);
				r.buffer.putFloat(h.colorTint.y);
				r.buffer.putFloat(h.colorTint.z);
			}
		});
		blockSide = new BlockRenderer<>(3 * 4 + 2 * 4 + 3 * 4, 4, 15000, (s, r) -> {
			for (int i = 0; i < r.verticesPerObject; i++) {
//				r.buffer.putFloat(4.65783f).putFloat(4.65483f).putFloat(4.65783f).putFloat(4.65883f).putFloat(4.65083f).putFloat(4.65783f).putFloat(4.62783f).putFloat(2.65783f);
//				r.buffer.putInt(Float.floatToRawIntBits(4.65783f)).putInt(Float.floatToRawIntBits(4.65483f)).putInt(Float.floatToRawIntBits(4.65783f)).putInt(Float.floatToRawIntBits(4.65883f)).putInt(Float.floatToRawIntBits(4.65083f)).putInt(Float.floatToRawIntBits(4.65783f)).putInt(Float.floatToRawIntBits(4.62783f)).putInt(Float.floatToRawIntBits(2.65783f));
				int x = (s.side + i / 2) % 6, y = (i + 1) % 4 / 2;
				Hexagon hex = y == 0 ? s.block.up : s.block.down;
				r.buffer.putFloat(s.block.blockColumn.points[x].x * hex.globalScale + Main.main.camera.position.getEucl().x * (1f - hex.globalScale));
				r.buffer.putFloat(s.block.blockColumn.points[x].y * hex.globalScale);
				r.buffer.putFloat(s.block.blockColumn.points[x].z * hex.globalScale);
				r.buffer.putFloat((x + 6 - s.side) % 6);
				r.buffer.putFloat(y);
				r.buffer.putFloat(hex.colorTint.x);
				r.buffer.putFloat(hex.colorTint.y);
				r.buffer.putFloat(hex.colorTint.z);
			}
		});
	}
	
	
	private final int vertexSizeInBytes;
	private final int objectSizeInBytes;
	private final int verticesPerObject;
	private final int numberOfObjects;
	private final int bufferSize;
	private final int numIndices;
	private final int vaoID, vboID, iboID;
	private final BiConsumer<T, BlockRenderer<T>> submitFunc;
	
	private ByteBuffer buffer;
	private int indicesUsed = 0;
	
	public BlockRenderer(int vertexSizeInBytes, int verticesPerObject, int numberOfObjects, BiConsumer<T, BlockRenderer<T>> sumbitFunc) {
		this.vertexSizeInBytes = vertexSizeInBytes;
		this.verticesPerObject = verticesPerObject;
		this.objectSizeInBytes = verticesPerObject * vertexSizeInBytes;
		this.numberOfObjects = numberOfObjects;
		this.bufferSize = numberOfObjects * this.objectSizeInBytes;
		this.numIndices = numberOfObjects * (verticesPerObject - 2) * 3;
		this.submitFunc = sumbitFunc;
		vaoID = GL30.glGenVertexArrays();
		vboID = GL15.glGenBuffers();
		iboID = numIndices > 0 ? GL15.glGenBuffers() : -1;
		Loader.theLoader.vaos.add(vaoID);
		Loader.theLoader.vbos.add(vboID);
		if (iboID != -1) Loader.theLoader.vbos.add(iboID);
		
		GL30.glBindVertexArray(vaoID);
		buffer = BufferUtils.createByteBuffer(bufferSize);
		GL15.glBindBuffer(GL15.GL_ARRAY_BUFFER, vboID);
		GL15.glBufferData(GL15.GL_ARRAY_BUFFER, bufferSize, GL15.GL_STREAM_DRAW);
		GL20.glEnableVertexAttribArray(0);
		GL20.glEnableVertexAttribArray(1);
		GL20.glEnableVertexAttribArray(2);
		if (iboID == -1) {
			GL30.glVertexAttribIPointer(0, 3, GL11.GL_INT, vertexSizeInBytes, 0); // position
			GL20.glVertexAttribPointer(1, 3, GL11.GL_FLOAT, false, vertexSizeInBytes, 3 * 4); // colorTint
			GL30.glVertexAttribIPointer(2, 1, GL11.GL_INT, vertexSizeInBytes, 6 * 4); // sides
		} else {
			GL20.glVertexAttribPointer(0, 3, GL11.GL_FLOAT, false, vertexSizeInBytes, 0); // position
			GL20.glVertexAttribPointer(1, 2, GL11.GL_FLOAT, false, vertexSizeInBytes, 3 * 4); // texCoords
			GL20.glVertexAttribPointer(2, 3, GL11.GL_FLOAT, false, vertexSizeInBytes, 5 * 4); // colorTint
		}
		
		if (iboID != -1) {
			ShortBuffer indices = BufferUtils.createShortBuffer(numIndices);
			for (short i = 0; i < numberOfObjects; i++) {
				for (short j = 0; j < verticesPerObject - 2; j++) {
					indices.put((short) (verticesPerObject * i));
					indices.put((short) (verticesPerObject * i + j + 1));
					indices.put((short) (verticesPerObject * i + j + 2));
				}
			}
			indices.flip();
			
			GL15.glBindBuffer(GL15.GL_ELEMENT_ARRAY_BUFFER, iboID);
			GL15.glBufferData(GL15.GL_ELEMENT_ARRAY_BUFFER, indices, GL15.GL_STATIC_DRAW);
		}
	}
	
	/**	Usage: begin, submit ..., end, render */
	public void submit(T t) {
		if (buffer.remaining() < objectSizeInBytes) flush();
		
		submitFunc.accept(t, this);
	}
	
	public void flush() {
		end();
		render();
		begin();
	}
	
	public long time = 0, totalTime = 0;
	public void begin() {
		buffer.clear();
	}
	
	public void end() {
		if (iboID != -1) indicesUsed = buffer.position() * 12 / objectSizeInBytes;
		else indicesUsed = buffer.position() / objectSizeInBytes;
		buffer.flip();
		GL15.glBindBuffer(GL15.GL_ARRAY_BUFFER, vboID);
		GL15.glBufferData(GL15.GL_ARRAY_BUFFER, buffer, GL15.GL_STREAM_DRAW);
	}
	
	public void render() {
		GL30.glBindVertexArray(vaoID);
		if (iboID != -1) GL15.glBindBuffer(GL15.GL_ELEMENT_ARRAY_BUFFER, iboID);
		GL13.glActiveTexture(GL13.GL_TEXTURE0);
		GL11.glBindTexture(GL11.GL_TEXTURE_2D, Hexagon.model.getTextureID());
		
		if (iboID != -1) GL11.glDrawElements(GL11.GL_TRIANGLES, indicesUsed, GL11.GL_UNSIGNED_SHORT, 0);
		else GL11.glDrawArrays(GL11.GL_POINTS, 0, indicesUsed);
	}
	
}
