package hexagon.util;

import org.lwjgl.util.vector.Vector3f;

public class Vec3f {
	
	private Vector3f eucl;
	private CylVec3 cyl;
	
	public Vec3f() {
		cyl = new CylVec3();
		eucl = new Vector3f();
	}
	
	public Vec3f(Vector3f vec) {
		eucl = vec;
		euclToCylCoords(vec, cyl = new CylVec3());
	}
	
	public Vec3f(CylVec3 vec) {
		cyl = vec;
		cylToEuclCoords(vec, eucl = new Vector3f());
	}
	
	public void setEucl(Vector3f src) {
		eucl.x = src.x;
		eucl.y = src.y;
		eucl.z = src.z;
		euclToCylCoords(eucl, cyl);
	}
	
	public void setEucl(float x, float y, float z) {
		eucl.x = x;
		eucl.y = y;
		eucl.z = z;
		euclToCylCoords(eucl, cyl);
	}
	
	public void setCyl(CylVec3 src) {
		cyl.x = src.x;
		cyl.r = src.r;
		cyl.rv = src.rv;
		cylToEuclCoords(cyl, eucl);
	}
	
	public void setCyl(float x, double r, double v) {
		cyl.x = x;
		cyl.r = (float) r;
		cyl.rv = (float) (r * v);
		cylToEuclCoords(cyl, eucl);
	}
	
	public Vector3f getEucl() {
		return eucl;
	}
	
	public CylVec3 getCyl() {
		return cyl;
	}
	
	public void updateEucl() {
		double circumf = Math.PI * 2 * cyl.r;
		cyl.rv %= circumf;
		if (cyl.rv < 0) cyl.rv += circumf;
		cylToEuclCoords(cyl, eucl);
	}
	
	public void updateCyl() {
		euclToCylCoords(eucl, cyl);
	}
	
	/** @param src where x = x, r = radius, v = angle */
	public static void cylToEuclCoords(CylVec3 src, Vector3f dest) {
		dest.x = src.x;
		dest.y = (float) (src.r * Math.sin(src.rv / src.r));
		dest.z = (float) (src.r * Math.cos(src.rv / src.r));
	}
	
	/** @param dest where x = x, y = radius, z = angle */
	public static void euclToCylCoords(Vector3f src, CylVec3 dest) {
		dest.x = src.x;
		dest.r = (float) Math.hypot(src.y, src.z);
		dest.rv = (float) Math.atan2(src.y, src.z) * dest.r;
	}
	
}
