package hexagon.world;

import java.util.Arrays;
import java.util.function.Consumer;

public class Chunk {
	/** Number of blocks in a chunk in one dimension (eg. 4 means 4x4 blocks) */
	public static final int chunkSize = 16;
	/** Number of chunks in a ring */
	public static final int ringSize = 16;
	
	protected World world;
	protected final BlockColumn[] blockCols = new BlockColumn[chunkSize * chunkSize];
	public final int x, z;
	
	public Chunk(int x, int z, Consumer<Chunk> generator, World world) {
		this.x = x;
		this.z = z;
		this.world = world;
		
		generator.accept(this);
	}
	
	public Block getBlock(int x, int y, int z) {
		return blockCols[x + z * chunkSize].getBlock(y);
	}
	
	public void setBlock(int x, int y, int z, Block block) {
		Block old = blockCols[x + z * chunkSize].getBlock(y);
		if (old != null) old.onRemove();
		blockCols[x + z * chunkSize].setBlock(block, y);
	}
	
	public void updateBlock(int x, int y, int z) {
		blockCols[x + z * chunkSize].getBlock(y).onUpdate();
	}
	
	public void onRemove() {
		for (BlockColumn c : blockCols)
			c.onRemove();
		Arrays.fill(blockCols, null);
	}
	
}
